

/* ***********************************************************
    PART ONE - Defining the script as an XSI Plugin
*/
//  So that XSI recognises and loads our script, we write 
//  it as a 'Self-Installing plugin'.
//  Of the many things a plugin allows us to do, we will use 
//  the following features :
//          * Map a function to command name
//          * Trigger code on an event
//          * Create menu item
//  
//  An XSI plugin is always defined by a function called 
//  XSILoadPlugin. If your plugin does not have this, XSI will
//  not recognise it.
function XSILoadPlugin(in_reg)
{
    //  in_reg is a variable given to us by XSI. Through this 
    //  we can define what the plugin can do
    
    //  Define a plugin name, this usually refers to the filename. This 
    //  is a collective name for all the file contents
    in_reg.Name = "Animation_PoseBalls";
    
    //  We define a command name which is like a shortcut to 
    //  a function within this file
    in_reg.RegisterCommand("PoseBallsUI");
    
    //  We define an event which triggers a function each time 
    //  something is selected. 
    in_reg.RegisterEvent("PoseBalls_SelectionEvent", // Name of function to run
                         siOnSelectionChange);       // Event type
    
    //  we add a menu to the XSI UI to make it easy to access 
    //  the tool
    in_reg.RegisterMenu(siMenuTbAnimateCreateCharacterID,  // Location to place the menu item
                        "PoseBalls");                      // Menu Item name
    
}







/* ***********************************************************
    PART TWO - Adding the menu item
*/
//  Start by adding the menu item. This will make it easier to 
//  trigger the tool as its written.
//  Notice the function name is the same as that which we declared
//  except for '_Init.' This defines it as the menu 'Initialisation'
//  function and further defines any submenu items.
function PoseBalls_Init(ctxt)
{
    // the ctxt variable given to us by XSI allows us
    // to add the menu item. AddCallbackItem  tells 
    // XSI to call a function when the 'Pose Balls' 
    // menu item is clicked.
    var menu = ctxt.source;

    menu.AddCallbackItem( "Pose Balls",            // Label for menu item
                          "PoseBallsUI_Execute" ); // Name of function to call
}









/* ***********************************************************
    PART THREE - Creating the visual tool
*/
//  As with the menu, notice that our function ends with '_Execute'. This
//  tells XSI this is the executable code for the command.
function PoseBallsUI_Execute()
{
    //  The UI for the tool uses a property, so check if one already exists,
    //  if it does, we remove it and re-create it. 
    //  Note that in Jscript, 'null' is like saying 'nothing'
    if ( ActiveSceneRoot.Properties("PoseBalls") != null )
        DeleteObj(ActiveSceneRoot.Properties("PoseBalls"));
    
    //  Add a new property
    var poseball_prop = ActiveSceneRoot.AddProperty("CustomProperty",  // Property Type
                                                    false           ,  // Branch, whether its children inherit the property
                                                    "PoseBalls"     ); // Name of the property
    
    //  Get the property layout as a variable
    var poseball_ui = poseball_prop.PPGLayout;
    
    //  Create an array to add the buttons to, this makes it easy to
    //  cycle the buttons setting their properties together
    var buttons = [];
    
    //  We add a series of layout groups and add the relevant buttons. Each button is 
    //  given a label, and a function name.
    poseball_ui.AddGroup("Create Pose Balls")
        buttons.push(poseball_ui.AddButton( "CreatePoseBall"   ,        // Function to call when clicked
                                            "Create Pose Ball"));       // Label for button
                                            
        buttons.push(poseball_ui.AddButton( "CreateResetPoseBall"    ,  // Function to call when clicked
                                            "Create Reset Pose Ball")); // Label for button
    poseball_ui.EndGroup();
    
    
    poseball_ui.AddGroup("Remove Pose Balls");
        buttons.push(poseball_ui.AddButton( "RemovePoseBall",          // Function to call when clicked
                                            "Remove Pose Ball"));       // Label for button
    poseball_ui.EndGroup();
    
    
    poseball_ui.AddGroup("Activate/Deactivate Options")
        buttons.push(poseball_ui.AddButton( "ActivatePoseBall",         // Function to call when clicked
                                            "Activate Pose Ball"));     // Label for button
                                            
        buttons.push(poseball_ui.AddButton( "DeactivatePoseBall",       // Function to call when clicked
                                            "Deactivate Pose Ball"));   // Label for button
    poseball_ui.EndGroup();
    
    
    //  We now set the buttons width
    for ( var i = 0; i < buttons.length; i++ )
    {
        buttons[i].SetAttribute(siUICX,  // Attribute to set ( width )
                                200);    // Value to set to
    }
    
    //  We then have to store all the code we want the buttons to run 
    //  within the custom property, and define the language of the code.
    //  Notice that each function has the OnClicked, this tells XSI to 
    //  trigger the function when the button is clicked.
    poseball_ui.Language = "Jscript";
    poseball_ui.Logic = CreatePoseBall_OnClicked.toString()+
                        RemovePoseBall_OnClicked.toString()+
                        ActivatePoseBall_OnClicked.toString()+
                        DeactivatePoseBall_OnClicked.toString()+
                        CreateResetPoseBall_OnClicked.toString();
    
    //  Tell xsi to show the property window
    InspectObj(poseball_prop);
}






/* ***********************************************************
    PART FOUR - Declaring the button code
*/
//  Notice that each function in this section ends with
//  OnClicked. This defines to XSI that these functions
//  are linked to the button states.



/* *************
    Create Standard Pivot
*/
function CreatePoseBall_OnClicked()
{
    //  Determine which object we're making a poseball for. If the user 
    //  does not have something selected, then we prompt
    //  for a selection
    if ( selection.Count < 1 )
    {
        //  PickElement is an XSI command which puts the user
        //  into a selection state where they are prompted
        //  to select an object
        var user_choice = PickElement();
    
        //  If the user did not left click an object, we exit
        if ( !user_choice.Value("ButtonPressed") )
            return;
        
        //  Get the selected object as a variable
        var target_object = user_choice.Value("PickedElement");
    }
    else
    {
        //  We just use the object the user already had selected
        var target_object = selection(0);
    }
    
    
    //  to prevent windows appearing during the build, we store the users
    //  popup settings, then disable pop-ups
    var user_setting = GetValue("preferences.Interaction.autoinspect");
    SetValue("preferences.Interaction.autoinspect", false);
    
    
    //  To keep the scene clean, we store all the pivots under a single 
    //  parent. If this parent does not exist, we create it
    var poseball_root = ActiveSceneRoot.FindChild("PoseBalls");
    
    if ( poseball_root == null )
        poseball_root = ActiveSceneRoot.AddNull("PoseBalls");
    
    
    //  Create the pivot object, and set its shape properties to be
    //  a simple and smooth sphere
    var poseball = poseball_root.AddGeometry("Sphere",                         // Object type
                                             "NurbsSurface",                   // Geometry type ( polymesh, nurbs etc )
                                             target_object.Name + "_pivot");   // Name for the obeject
    
    poseball.subdivu = 4;
    poseball.subdivv = 2;
    poseball.radius  = 0.15;
    
    
    
    //  to define it as a poseball, we give it a property with a specific name and set 
    //  of parameters storing important infomation.
    var poseball_settings = poseball.AddProperty("CustomProperty", false, "PoseBallOptions");
    
    //  Add the core parameters. I find that aligning the variables
    //  between different lines makes the code much easier to read
    //  Note :
    //  AddParameter3( name_of_parameter,
    //                 type_of_parameter,
    //                 default_parameter_value )
    var target_param = poseball_settings.AddParameter3("target", siString, target_object.FullName);
    var active_param = poseball_settings.AddParameter3("Active", siBool  , false                 );
    var in_use_param = poseball_settings.AddParameter3("InUse" , siBool  , false                 );
    
    //  We then add some parameters which relate to color, so the user can define
    //  the visual style of the pivots
    var active_col_r_param = poseball_settings.AddParameter3("ActiveRcol", siDouble, 1);
    var active_col_g_param = poseball_settings.AddParameter3("ActiveGcol", siDouble, 0);
    var active_col_b_param = poseball_settings.AddParameter3("ActiveBcol", siDouble, 0);
    
    var inactive_col_r_param = poseball_settings.AddParameter3("InActiveRcol", siDouble, 0  );
    var inactive_col_g_param = poseball_settings.AddParameter3("InActiveGcol", siDouble, 0.5  );
    var inactive_col_b_param = poseball_settings.AddParameter3("InActiveBcol", siDouble, 1);
    
    
    //  We give the poseball a Shader, so we can control its color
    ApplyShader("Material\\Constant",           // Shader Type to add
                poseball,                       // Object to apply shader to
                poseball.Name+"_shader",        // Name of the shader
                false,                          // Apply to rest of hierachy or not
                siLetLocalMaterialsOverlap);    // Allow overlapping materials
    
    var shader = poseball.Material.Shaders(0);
    
    //  We then set an expression on the colour parameters of the shader, this uses
    //  the parameters we previously created, and determines which to use depending on
    //  whether the 'Active' parameter is true or false
    //  Note, Expression :
    //    (pivot_in_use_condition, do_if_condition_is_true, do_if_condition_is_false) + ( active_overide )
    shader.NestedObjects("color").Parameters("red"  ).AddExpression("cond(" + in_use_param.FullName + ">0," + active_col_r_param.FullName + " ," + inactive_col_r_param.FullName + ")+("+active_param.FullName+ "*-1 + 1)");
    shader.NestedObjects("color").Parameters("green").AddExpression("cond(" + in_use_param.FullName + ">0," + active_col_g_param.FullName + " ," + inactive_col_g_param.FullName + ")+("+active_param.FullName+ "*-1 + 1)");
    shader.NestedObjects("color").Parameters("blue" ).AddExpression("cond(" + in_use_param.FullName + ">0," + active_col_b_param.FullName + " ," + inactive_col_b_param.FullName + ")+("+active_param.FullName+ "*-1 + 1)");
    
    //  Constrain the the pivot to the object so that it always moves with it
    poseball.Kinematics.AddConstraint("pose", target_object);
    
    //  Set XSI to constrain comp mode, so the user can move
    //  the pivot
    SetUserPref("SI3D_CONSTRAINT_COMPENSATION_MODE", 1);
    
    //  Select the pivot for the user
    Selection.SetAsText(poseball.FullName);
    
    //  Restore the users pop-up settings
    SetValue("preferences.Interaction.autoinspect", user_setting, null);
    
    //  return the poseball variable
    return poseball
}




/* *************
    Create Reset Pivot
*/
function CreateResetPoseBall_OnClicked()
{
    //  We start by creating a standard pivot, using the code we 
    //  have already written
    var poseball = CreatePoseBall_OnClicked()
    
    //  Get the options property
    var options = poseball.Properties("PoseBallOptions")
    
    //  We need to add the reset parameter to the pivot, so it is not
    //  seen as a standard pivot
    options.AddParameter3("Reset", // Parameter name
                          siBool,  // Parameter type
                          true)    // Default Parameter value
    
    //  Get the pivots target object
    var target_object = ActiveSceneRoot.FindChild(options.Parameters("target").Value)
    
    //  We visually distinguish a reset pivot by using a line drawn from
    //  the pivot to the target object.
    //  To draw a line, we have to give an array of x,y,z,w values which determine
    //  the worldspace placement of the curve points.
    var crv_marker = poseball.AddNurbsCurve([target_object.Kinematics.Global.PosX.value,   // Point 1 : X
                                             target_object.Kinematics.Global.PosY.value,   // Point 1 : Y
                                             target_object.Kinematics.Global.PosZ.value,1, // Point 1 : Z,W
                                             poseball .Kinematics.Global.PosX.value,       // Point 2 : X
                                             poseball .Kinematics.Global.PosY.value,       // Point 2 : Y
                                             poseball .Kinematics.Global.PosZ.value,1],    // Point 2 : Z,W
                                             null,                                         // Knots (null = derive from points)
                                             false,                                        // Closed curve ( circular ), or open ( line )
                                             1);                                           // Degree of curvature, 1 = straight, 3 = smooth curve
    
    //  We need to lock the curve points to the poseball and target.
    //  We do this by adding a cluster, then defining a specific
    //  'center reference' for that cluster.
    var cls1 = crv_marker.ActivePrimitive.Geometry.AddCluster(siVertexCluster,  // Cluster type to add ( vertex/polygon etc )
                                                              "markerCls"    ,  // Name of cluster
                                                              [0]            ); // Point to use
                                                              
    var cls2 = crv_marker.ActivePrimitive.Geometry.AddCluster(siVertexCluster,  // Cluster type to add ( vertex/polygon etc )
                                                              "markerCls"    ,  // Name of cluster
                                                              [1]            ); // Point to use
    
    cls1.CenterReference = target_object;
    cls2.CenterReference = poseball;
    
    //  Make sure the object is selected for the user
    Selection.SetAsText(poseball.FullName);
}



/* *************
    Remove Pivot
*/
function RemovePoseBall_OnClicked()
{
    //  Define a variable to watch 
    var user_interaction = true;
    
    //  While the variable is true, keep asking for user inputs
    while ( user_interaction )
    {
        //  Find out which poseball to remove
        var user_choice = PickElement();
        
        //  If no selection is made ( right click ), exit
        if ( !user_choice.Value("ButtonPressed") )
            return;
        
        //  Get the selection
        var object = user_choice.Value("PickedElement");
        
        //  Check the user has selected a valid poseball
        var poseball_options = object.Properties("PoseBallOptions");

        if (poseball_options == null )
        {
            // Log a message
            Logmessage("This object is not a poseball.");
            
            //  Restart the cycle asking for another object
            continue;
        }
        
        //  Delete the pivot
        DeleteObj("B:" + target_object);
    }
}




/* *************
    Activates a Pivot
*/
function ActivatePoseBall_OnClicked()
{
    //  Define a variable to watch 
    var user_interaction = true;
    
    //  While the variable is true, keep asking for user inputs
    while ( user_interaction )
    {
        //  Find out which poseball to activate
        var user_choice = PickElement();
        
        //  If no selection is made ( right click ), exit
        if ( !user_choice.Value("ButtonPressed") )
            return;
        
        //  Get the selection
        var object = user_choice.Value("PickedElement");
        
        //  Check the user has selected a valid poseball
        var poseball_options = object.Properties("PoseBallOptions");

        if (poseball_options == null )
        {
            // Log a message
            Logmessage("This object is not a poseball.");
            
            //  Restart the cycle asking for another object
            continue;
        }
        
        //  Set the active parameter for the pivot to be true
        poseball_options.Active = true;
    }
}





/* *************
    Deactivates a Pivot
*/
function DeactivatePoseBall_OnClicked()
{
    //  Define a variable to watch 
    var user_interaction = true;
    
    //  While the variable is true, keep asking for user inputs
    while ( user_interaction )
    {
        //  Find out which poseball to deactivate
        var user_choice = PickElement();
        
        //  If no selection is made ( right click ), exit
        if ( !user_choice.Value("ButtonPressed") )
            return;
        
        //  Get the selection
        var object = user_choice.Value("PickedElement");
        
        //  Check the user has selected a valid poseball
        var poseball_options = object.Properties("PoseBallOptions");

        if (poseball_options == null )
        {
            // Log a message
            Logmessage("This object is not a poseball.");
            
            //  Restart the cycle asking for another object
            continue;
        }
        
        //  Set the active parameter for the pivot to be false
        poseball_options.Active = false;
    }
}




/* ***********************************************************
    PART FIVE - The Selection Event
*/
//-- Notice our event function has the same name we wrote
//   in XSILoadPlugin, except it ends in _OnEvent
function PoseBalls_SelectionEvent_OnEvent()
{
    /*
        This selection event is triggered each time the user
        selects an object. If it is a 'poseball' object, then
        we need to set the pivot accordingly.
    */
    
    //  We use a try as a 'lazy' way of catching when the user selects
    //  properties/clusters/operators etc that are not 3d objects
    try
    {
        //  A poseball is always defined by a property named 'PoseBallOptions'
        //  so check for that
        poseball_options = selection(selection.Count-1).Properties("PoseBallOptions");
        
        //  If this is not a poseball, we may need to reset the transform settings
        if ( poseball_options == null )
        {
            //  Find out what the last target was. GetGlobal & SetGlobal are XSI commands
            //  that allow us to store variables within the scene session rather than code
            var last_pivot = dictionary.GetObject(GetGlobal("activeposeball"));
            var target_obj = dictionary.GetObject(last_pivot.Properties("PoseBallOptions").Parameters("target").Value);
            
            //  To prevent the pivot rotation settings used on this object
            //  we reset them to 'local' rotations
            if ( selection(selection.Count-1).Name != target_obj.Name)
            {
                SetTransientReferencePlane(Selection(0));
                SetUserPref ("3D_TRANSFO_EDITED_AXIS_CHANGED",7)
                SetUserPref ("3D_TRANSFO_REFERENTIAL_CHANGED",2);
            }
            
            //  We dont need to do anything more. 
            return;
        }
    }
    catch (error)
    {
        //  If we errored its because the selection was not a valid
        //  poseball selection, so we return
        return;
    }
    
    //  Get pivot object
    var selected_object = selection(selection.Count-1);
    
    
    //  Disable the previous poseball by settings its special
    //  property values
    var prev_poseball_name = GetGlobal("activeposeball");
    
    if  ( prev_poseball_name == null || prev_poseball_name == "" )
        prev_poseball_name = "unknown_pivot_name";
    
    
    var prev_poseball = dictionary.GetObject(prev_poseball_name, false);
    
    if ( prev_poseball != null )
            prev_poseball.Properties("PoseBallOptions").Parameters("InUse").value = false;
    
    
    //  Get the property from our object
    poseball_options = selected_object.Properties("PoseBallOptions");
    
    
    //  If this poseball is disabled, then we dont pivot from
    //  it. Instead, we exit and do nothing more.
    if ( !poseball_options.Parameters("Active").Value )
        return;
    
    
    //  Set the new pivot to be in use
    if ( poseball_options.Parameters("InUse") != null )
         poseball_options.Parameters("InUse").Value = true;
    
    
    
    //  Set the global variable to this new poseball
    //  allowing us to disable it when the user selects another pivot
    SetGlobal("activeposeball", selected_object.Name);
    
    
    
    //  We need to check if the poseball is a reset poseball, or a standard
    //  poseball type.
    if ( poseball_options.Parameters("Reset") == null )
    {
        //  We are dealing with a standard pivot
        
        //  Get the target object from the pivot
        var poseball              = selected_object;
        var poseball_name         = poseball.FullName;
        var target_object_name    = poseball_options.Parameters("target").Value;
        var target_object         = dictionary.GetObject(target_object_name);
        
        //  We switch the selection from the poseball
        //  to the animatable object
        Selection.Remove(selected_object);
        Selection.Add(target_object);
        
        //  We then set XSI to use the poseball as the 
        //  pivot point. Then set the manipulation to rotation.
        SetTransientReferencePlane(poseball_name);
        RotationTool();
        SetUserPref ("3D_TRANSFO_EDITED_AXIS_CHANGED",7)
        SetUserPref ("3D_TRANSFO_REFERENTIAL_CHANGED",3);
        
    }
    else
    {
        //  we are dealing with a reset pivot
        
        //  Get the target object from the poseball
        var target_object_name = poseball_options.Parameters("target").Value;
        var target_object      = dictionary.GetObject(target_object_name);
        
        //  Select the poseball target object
        Selection.Remove(selected_object)
        Selection.Add(target_object);
        
        //  Set the manipulation method to rotation
        //  from the actual object
        RotationTool();
        SetUserPref ("3D_TRANSFO_EDITED_AXIS_CHANGED",7)
        SetUserPref ("3D_TRANSFO_REFERENTIAL_CHANGED",2);
        
    }
}
